example_ecoindex_not_found = {
    "description": "Not found",
    "content": {
        "application/json": {
            "example": {
                "detail": (
                    "Analysis e9a4d5ea-b9c5-4440-a74a-cac229f7d672 "
                    "not found for version v1"
                )
            }
        }
    },
}

example_file_not_found = {
    "description": "Not found",
    "content": {
        "application/json": {
            "example": {
                "detail": (
                    "File at path screenshots/v0/"
                    "550cdf8c-9c4c-4f8a-819d-cb69d0866fe1.webp does not exist."
                )
            }
        }
    },
}

example_page_listing_empty = {
    "description": "Empty page",
    "content": {
        "application/json": {
            "example": {
                "items": [],
                "total": 0,
                "page": 1,
                "size": 10,
            }
        }
    },
}

example_daily_limit_response = {
    "description": "You have reached the daily limit",
    "content": {
        "application/json": {
            "example": {
                "detail": {
                    "daily_limit_per_host": 1,
                    "limit": 1,
                    "host": "www.ecoindex.fr",
                    "latest_result": {
                        "width": 1920,
                        "height": 1080,
                        "size": 107.178,
                        "requests": 6,
                        "score": 87,
                        "water": 1.89,
                        "date": "2023-01-05T12:06:57",
                        "id": "be8c3612-545f-4e72-8880-13b8db74ff6e",
                        "version": 1,
                        "initial_ranking": 1,
                        "url": "https://www.ecoindex.fr",
                        "nodes": 201,
                        "grade": "A",
                        "ges": 1.26,
                        "ecoindex_version": "5.4.1",
                        "page_type": None,
                        "host": "www.ecoindex.fr",
                        "initial_total_results": 1,
                    },
                    "message": (
                        "You have already reached the daily limit of 1 "
                        "requests for host www.ecoindex.fr today"
                    ),
                }
            }
        }
    },
}

example_daily_limit_response = {
    "description": "You have reached the daily limit",
    "content": {
        "application/json": {
            "example": {
                "detail": {
                    "daily_limit_per_host": 1,
                    "limit": 1,
                    "host": "www.ecoindex.fr",
                    "latest_result": {
                        "width": 1920,
                        "height": 1080,
                        "size": 107.178,
                        "requests": 6,
                        "score": 87,
                        "water": 1.89,
                        "date": "2023-01-05T12:06:57",
                        "id": "be8c3612-545f-4e72-8880-13b8db74ff6e",
                        "version": 1,
                        "initial_ranking": 1,
                        "url": "https://www.ecoindex.fr",
                        "nodes": 201,
                        "grade": "A",
                        "ges": 1.26,
                        "ecoindex_version": "5.4.1",
                        "page_type": None,
                        "host": "www.ecoindex.fr",
                        "initial_total_results": 1,
                    },
                    "message": (
                        "You have already reached the daily limit of 1 "
                        "requests for host www.ecoindex.fr today"
                    ),
                }
            }
        }
    },
}



class EcoindexScraperException(Exception):
    pass


class EcoindexScraperStatusException(EcoindexScraperException):
    def __init__(self, url: str, status: int, message: str):
        self.message = message
        self.url = url
        self.status = status

    pass



from abc import ABC, abstractmethod
from csv import DictWriter
from json import dump
from os import makedirs
from os.path import dirname, exists

from ecoindex.models import ExportFormat, Language, Result
from yaml import safe_load as load_yaml


def create_folder(path: str) -> None:
    if not exists(path):
        makedirs(path)


class File(ABC):
    def __init__(
        self,
        filename: str,
        results: list[Result],
        export_format: ExportFormat | None = ExportFormat.csv,
    ):
        self.filename = filename
        self.results = results
        self.export_format = export_format

    @abstractmethod
    def write(self) -> None:
        pass


class CsvFile(File):
    def write(self) -> None:
        headers = self.results[0].__dict__

        with open(self.filename, "w") as fp:
            writer = DictWriter(fp, fieldnames=headers)

            writer.writeheader()
            for ecoindex in self.results:
                writer.writerow(ecoindex.__dict__)


class JsonFile(File):
    def write(self) -> None:
        with open(self.filename, "w") as fp:
            dump(
                obj=[ecoindex.__dict__ for ecoindex in self.results],
                fp=fp,
                indent=4,
                default=str,
            )


def write_results_to_file(
    filename: str,
    results: list[Result],
    export_format: ExportFormat | None = ExportFormat.csv,
) -> None:
    if export_format == ExportFormat.csv:
        CsvFile(filename=filename, results=results, export_format=export_format).write()

    if export_format == ExportFormat.json:
        JsonFile(
            filename=filename, results=results, export_format=export_format
        ).write()


def write_urls_to_file(file_prefix: str, urls: list[str]) -> None:
    tmp_input_folder = "/tmp/ecoindex-cli/input"
    create_folder(tmp_input_folder)
    with open(
        file=f"{tmp_input_folder}/{file_prefix}.csv", mode="w"
    ) as input_urls_file:
        for url in urls:
            input_urls_file.write(f"{str(url).strip()}\n")


def get_translations(language: Language) -> dict:
    filename = f"{dirname(__file__)}/cli_translations/{language.value}.yml"
    with open(filename) as fp:
        return load_yaml(fp)



class EcoindexContentTypeError(Exception):
    pass


class EcoindexHostUnreachable(Exception):
    pass


class EcoindexPageNotFound(Exception):
    pass


class EcoindexStatusError(Exception):
    pass


class EcoindexTimeout(Exception):
    pass



from ecoindex.compute.ecoindex import (
    compute_ecoindex,
    get_ecoindex,
    get_grade,
    get_greenhouse_gases_emmission,
    get_quantile,
    get_score,
    get_water_consumption,
)

__all__ = [
    "compute_ecoindex",
    "get_ecoindex",
    "get_grade",
    "get_greenhouse_gases_emmission",
    "get_quantile",
    "get_score",
    "get_water_consumption",
]



import json
import os
from datetime import datetime
from time import sleep
from uuid import uuid4

from ecoindex.compute import compute_ecoindex
from ecoindex.exceptions.scraper import EcoindexScraperStatusException
from ecoindex.models.compute import PageMetrics, Result, ScreenShot, WindowSize
from ecoindex.models.scraper import MimetypeAggregation, RequestItem, Requests
from ecoindex.utils.screenshots import convert_screenshot_to_webp, set_screenshot_rights
from playwright.async_api import async_playwright
from playwright_stealth import stealth_async
from typing_extensions import deprecated


class EcoindexScraper:
    def __init__(
        self,
        url: str,
        window_size: WindowSize = WindowSize(width=1920, height=1080),
        wait_before_scroll: float = 1,
        wait_after_scroll: float = 1,
        screenshot: ScreenShot | None = None,
        screenshot_uid: int | None = None,
        screenshot_gid: int | None = None,
        page_load_timeout: int = 20,
    ):
        self.url = url
        self.window_size = window_size
        self.wait_before_scroll = wait_before_scroll
        self.wait_after_scroll = wait_after_scroll
        self.screenshot = screenshot
        self.screenshot_uid = screenshot_uid
        self.screenshot_gid = screenshot_gid
        self.page_load_timeout = page_load_timeout
        self.all_requests = Requests()
        self.now = datetime.now()
        self.har_temp_file_path = (
            f"/tmp/ecoindex-{self.now.strftime('%Y-%m-%d-%H-%M-%S-%f')}-{uuid4()}.har"
        )

    @deprecated("This method is useless with new version of EcoindexScraper")
    def init_chromedriver(self):
        return self

    async def get_page_analysis(self) -> Result:
        page_metrics = await self.scrap_page()
        ecoindex = await compute_ecoindex(**page_metrics.model_dump())

        return Result(
            **ecoindex.model_dump(),
            **self.window_size.model_dump(),
            **page_metrics.model_dump(),
            date=self.now,
            url=self.url,
        )

    async def get_all_requests(self) -> list[RequestItem]:
        return self.all_requests.items

    async def get_requests_by_category(self) -> MimetypeAggregation:
        return self.all_requests.aggregation

    async def scrap_page(self) -> PageMetrics:
        async with async_playwright() as p:
            browser = await p.chromium.launch()
            self.page = await browser.new_page(
                record_har_path=self.har_temp_file_path,
                screen=self.window_size.model_dump(),
                ignore_https_errors=True,
            )
            await stealth_async(self.page)
            response = await self.page.goto(self.url)
            await self.check_page_response(response)

            await self.page.wait_for_load_state()
            sleep(self.wait_before_scroll)
            await self.generate_screenshot()
            await self.page.keyboard.press("ArrowDown")
            await self.page.evaluate(
                "window.scrollTo({ top: document.body.scrollHeight, behavior: 'smooth' })"
            )
            sleep(self.wait_after_scroll)
            total_nodes = await self.get_nodes_count()
            await self.page.close()
            await browser.close()

        await self.get_requests_from_har_file()

        return PageMetrics(
            size=self.all_requests.total_size / 1000,
            nodes=total_nodes,
            requests=self.all_requests.total_count,
        )

    async def generate_screenshot(self) -> None:
        if self.screenshot and self.screenshot.folder and self.screenshot.id:
            await self.page.screenshot(path=self.screenshot.get_png())
            await convert_screenshot_to_webp(self.screenshot)
            await set_screenshot_rights(
                screenshot=self.screenshot,
                uid=self.screenshot_uid,
                gid=self.screenshot_gid,
            )

    async def get_requests_from_har_file(self):
        with open(self.har_temp_file_path, "r") as f:
            trace = json.load(f)
            aggregation = self.all_requests.aggregation.model_dump()

            for entry in trace["log"]["entries"]:
                url = entry["request"]["url"]
                mime_type = entry["response"]["content"]["mimeType"]
                category = await MimetypeAggregation.get_category_of_resource(mime_type)
                aggregation[category]["total_count"] += 1
                size = self.get_request_size(entry)
                aggregation[category]["total_size"] += size
                self.all_requests.total_count += 1
                self.all_requests.total_size += size
                self.all_requests.items.append(
                    RequestItem(
                        url=url,
                        mime_type=mime_type,
                        status=entry["response"]["status"],
                        size=size,
                        category=category,
                    )
                )
            self.all_requests.aggregation = MimetypeAggregation(**aggregation)
        os.remove(self.har_temp_file_path)

    async def get_nodes_count(self) -> int:
        nodes = await self.page.locator("*").all()
        svgs = await self.page.locator("//*[local-name()='svg']//*").all()

        return len(nodes) - len(svgs)

    def get_request_size(self, entry) -> int:
        if entry["response"]["_transferSize"] != -1:
            return entry["response"]["_transferSize"]
        else:
            return len(json.dumps(entry["response"]).encode("utf-8"))

    async def check_page_response(self, response) -> None:
        if response and response.status != 200:
            raise EcoindexScraperStatusException(
                url=self.url,
                status=response.status,
                message=response.status_text,
            )
        headers = response.headers
        content_type = next(
            (value for key, value in headers.items() if key.lower() == "content-type"),
            None,
        )
        if content_type and "text/html" not in content_type:
            raise TypeError(
                {
                    "mimetype": content_type,
                    "message": (
                        "This resource is not "
                        "a standard page with mimeType 'text/html'"
                    ),
                }
            )



from ecoindex.models.cli import (
    CliHost,
)
from ecoindex.models.compute import (
    Ecoindex,
    PageMetrics,
    PageType,
    Request,
    Result,
    ScreenShot,
    WebPage,
    WindowSize,
)
from ecoindex.models.enums import ExportFormat, Language, Version
from ecoindex.models.response_examples import (
    example_daily_limit_response,
    example_ecoindex_not_found,
    example_file_not_found,
    example_page_listing_empty,
)
from ecoindex.models.scraper import RequestItem, Requests
from ecoindex.models.sort import Sort

__all__ = [
    "CliHost",
    "Ecoindex",
    "example_daily_limit_response",
    "example_ecoindex_not_found",
    "example_file_not_found",
    "example_page_listing_empty",
    "ExportFormat",
    "Language",
    "PageMetrics",
    "PageType",
    "Request",
    "RequestItem",
    "Requests",
    "Result",
    "ScreenShot",
    "Sort",
    "Version",
    "WebPage",
    "WindowSize",
]
